@@ -1,4 +1,9 @@
 
+class NamespaceError(Exception):
+    """An error raised by the namespace package"""
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
 
 class Namespace:
     """
@@ -49,29 +54,25 @@ class Namespace:
         def offset(self, value):
             self.root.send_command('channel{0}:offset {1}'.format(self.index, value))
     """
-
     def __init__(self, cls):
         self.cls = cls
 
     def __get__(self, instance, owner):
-        if not instance:
+        if instance is None:
             return self.cls
 
-        name = self.cls.__name__ + 'cached'
-
-        # if we are already cached, use this version
-        if name in instance.__dict__:
-            return instance.__dict__[name]
-
         # create instance
         namespace = self.cls()
         # parent
         namespace.parent_namespace = instance
-        # find root, root has no root attribute
-        namespace.root = instance
-        while hasattr(namespace.root, 'root'):
-            namespace.root = namespace.root.root
-        setattr(instance, name, namespace)
+        # root. instance is root if it does not contain a root attribute
+        try:
+            namespace.root = instance.root
+        except AttributeError:
+            namespace.root = instance
+
+        # save it in the dictionary
+        instance.__dict__[self.cls.__name__] = namespace
         return namespace
 
     @classmethod
@@ -86,33 +87,37 @@ class Namespace:
                 self.cls = cls
                 self.count = count
 
+            def _raise_namespace_error(self, message):
+                """Raises a namespace error with a bit of additional information"""
+                raise NamespaceError("Namespace class '{}': {}".format(self.cls.__name__, message))
+
             def __get__(self, instance, owner):
-                if not instance:
+                if instance is None:
                     return self.cls
 
-                name = self.cls.__name__ + 'cached'
-
-                # if we are already cached, use this version
-                if name in instance.__dict__:
-                    return instance.__dict__[name]
-
                 # find root, root has no root attribute
-                root = instance
-                while hasattr(root, 'root'):
-                    root = root.root
+                try:
+                    root = instance.root
+                except AttributeError:
+                    root = instance
 
                 # determine number of instances
+
                 # if it is a string, check namespace container instance first, then root
+                if isinstance(self.count, str):
+                    value = getattr(instance, self.count, getattr(root, self.count, None))
+                    if value is None:
+                        self._raise_namespace_error("Could not find the field '{}' on namespace parent '{}' or namespace root '{}' (in that order)".format(
+                            self.count, type(instance).__name__, type(root).__name__
+                        ))
+                    self.count = value
+
+                # Validate that the given count is valid
                 if isinstance(self.count, int):
                     if self.count <= 0:
-                        raise Exception('Invalid value for count: {0}.'.format(self.count))
-                elif isinstance(self.count, str):
-                    value = getattr(instance, self.count, None)
-                    if value is None:
-                        value = getattr(root, self.count, 1)
-                    self.count = value
+                        self._raise_namespace_error('Namespace.repeat value must be a positive integer. Instead it was: {0}.'.format(self.count))
                 else:
-                    raise Exception('Invalid value for count: {0}.'.format(self.count))
+                    self._raise_namespace_error('Namespace.repeat value must be a positive integer. Instead it was: {0}.'.format(self.count))
 
                 # create instances
                 namespaces = [self.cls() for ii in range(self.count)]
@@ -120,7 +125,9 @@ class Namespace:
                     namespaces[ii].parent_namespace = instance
                     namespaces[ii].root = root
                     namespaces[ii].index = ii
-                setattr(instance, name, namespaces)
+                    
+                # save it in the dictionary
+                instance.__dict__[self.cls.__name__] = namespaces
                 return namespaces
 
         return NamespaceRepeat
